// Hilfsprogramm zum Finden der Konstanten fuer Drehscheibenabgaenge oder andere Stepperprogramme
// Nach dem Starten muss zuerst der Endschalter gefunden werden, danach koennen Werte im Monitor eingegeben 
// werden und der Stepper faehrt dort hin. Maximalwertevtl. editieren 

#include <AccelStepper.h>  // Stepper Bibliothek einbinden
#define HALFSTEP 8         // 8 Schritte pro Sequenz d.h. Halbschrittbetrieb 
                           // 28BYJ-48 funktioniert damit am Besten
// Motor pin am Arduino definieren
#define motorPin1  4     // IN1 on the ULN2003 driver 1
#define motorPin2  5     // IN2 on the ULN2003 driver 1
#define motorPin3  6     // IN3 on the ULN2003 driver 1
#define motorPin4  7     // IN4 on the ULN2003 driver 1

int endSchalter = 0;     // Variable fuer den Endschalter

// Steppertreiber initialisieren mit Pin Sequenz IN1-IN3-IN2-IN4
// damit AccelStepper mit 28BYJ-48 richtig funktioniert

AccelStepper stepper1(HALFSTEP, motorPin1, motorPin3, motorPin2, motorPin4);

// Arduino Programme haben alle die Struktur 
// void setup() die einmal zum Start durchlaufen wird
// void loop() , das dann endlos wiederholt wird

void setup() {
  pinMode(12, INPUT_PULLUP);  // Pin 12 als Eingang fuer Endschalter mit internem Widerstand definieren
  digitalWrite (12,HIGH);     // Sollte eigentlich nicht notwendig sein, aber schadet auch nicht
  
  Serial.begin(115200);      // Serielle Schnittstelle zum Laptop initialisieren,
                             // mit Tools->Serial Monitor kann man Meldungen an den Steuerrechner senden
  Serial.println("Programm zum Justieren der Drehscheibe");   // und gleich mal probieren
  Serial.println("Zahl eingeben und <send> anklicken"); 
                             // Uebrigens Monitor erst aktivieren, wenn der "Upload" des Programms erfolgt ist.
                             // Wenn man den Beginn des Programms verfolgen will, einfach die Reset-Taste
                             // am Arduino Board druecken. Ohne angeschlossenen Rechner gehen die Meldungen
                             // einfach "ins Leere"

  stepper1.setMaxSpeed(100.0);    // Max Geschwindigkeit, Beschleunigung und gesetzte Geschwindigkeit einstellen 
  stepper1.setAcceleration(200.0);// Zum Suchen des Endschalters langsam fahren
  stepper1.setSpeed(50);
  stepper1.setCurrentPosition(0); // Position definiert auf "0" setzen
  stepper1.moveTo(20000);         // Einen weiten Weg vorgeben, damit die Drehscheibe den Endschalter erreicht

  // Setup sieht eigentlich keine Schleifen vor, deshalb "kuenstlich" mit while konstruiert
  endSchalter = digitalRead(12);  // Endschalter einlesen, damit in der while-Schleife ein definierter Zustand entstejht
  while (endSchalter == HIGH){    // Diese Schleife durchlaufen, solange der Endschalter nicht erreicht ist.
    endSchalter = digitalRead(12);  // Endschalter bei jedem Durchlauf testen
    stepper1.run ();                // Besonderheit des Arduino, die Funktion muss haeufig aufgerufen werden (wie oft ?) 
  }
  // Endschalter wurde erreicht, while Schleife wird verlassen 
  stepper1.stop();                  // Stepper anhalten
  delay (1000);                     // Warten, bis er steht
  Serial.println ("Endschalter erreicht");
  stepper1.setMaxSpeed(1000.0);     // Nun Motor schneller machen fuer Normalbetrieb
  stepper1.setAcceleration(100.0);
  stepper1.setSpeed(200);
  stepper1.setCurrentPosition(0);   // Dies ist nun die endgueltige "Nullposition", von der aus alle Bewegungen
                                    // gezaehlt werden

}//--(end setup )---

void loop() {

  int position;
  
  if (Serial.available() > 0)  {          // Wenn keine Daten am seriellen Port, tu nix
    position=Serial.parseInt();             // Integer Zahl lesen
    while (Serial.available() > 0)          // Falls noch Zeichen im Eingangspuffer sind, loeschen
      Serial.read();
  
    if (position < 0) position = 0;         // Position auf 0...4076 bzw. 1 Umdrehung  begrenzen
    if (position > 4076) position = 4076;   // Diese Zahl experimentell aendern, sodass eine Umdrehung erreicht wird 
                                            // Zahl kann groesser sein, um die Schrittzahl fuer 360Grad zu ermitteln
    Serial.print ("Stepper faehrt nach: "); // Position ausgeben
    Serial.println (position);
    stepper1.moveTo(position);              // Motor starten
  }
  stepper1.run ();    // wie gesagt, muss haeufig aufgerufen werden...
}
